(*
 * This file is part of Barista.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

class type class_definition_mapper = object
  method class_definition : AccessFlag.for_class list -> Name.for_class -> Name.for_class option -> Name.for_class list -> Field.t list -> Method.t list -> Attribute.for_class list -> (AccessFlag.for_class list * Name.for_class * Name.for_class option * Name.for_class list * Field.t list * Method.t list * Attribute.for_class list)
  method class_flags : AccessFlag.for_class list -> AccessFlag.for_class list
  method class_name : Name.for_class -> Name.for_class
  method class_extends : Name.for_class option -> Name.for_class option
  method class_implements : Name.for_class -> Name.for_class
  method class_field : AccessFlag.for_field list -> Name.for_field -> Descriptor.for_field -> Attribute.for_field list -> Field.t
  method class_method : Method.t -> Method.t
  method class_attribute : Attribute.for_class -> Attribute.for_class
  method field_flags : AccessFlag.for_field list -> AccessFlag.for_field list
  method field_name : Name.for_field -> Name.for_field
  method field_descriptor : Descriptor.for_field -> Descriptor.for_field
  method field_attribute : Attribute.for_field -> Attribute.for_field
  method regular_method : AccessFlag.for_method list -> Name.for_method -> Descriptor.for_method -> Attribute.for_method list -> (AccessFlag.for_method list * Name.for_method * Descriptor.for_method * Attribute.for_method list)
  method regular_method_flags : AccessFlag.for_method list -> AccessFlag.for_method list
  method regular_method_name : Name.for_method -> Name.for_method
  method regular_method_descriptor : Descriptor.for_method -> Descriptor.for_method
  method regular_method_attribute : Attribute.for_method -> Attribute.for_method
  method constructor_method : AccessFlag.for_constructor list -> Descriptor.for_parameter list -> Attribute.for_method list -> (AccessFlag.for_constructor list * Descriptor.for_parameter list * Attribute.for_method list)
  method constructor_method_flags : AccessFlag.for_constructor list -> AccessFlag.for_constructor list
  method constructor_method_descriptor : Descriptor.for_parameter list -> Descriptor.for_parameter list
  method constructor_method_attribute : Attribute.for_method -> Attribute.for_method
  method initializer_method : bool -> Attribute.for_method list -> (bool * Attribute.for_method list)
  method initializer_method_strictfp : bool -> bool
  method initializer_method_attribute : Attribute.for_method -> Attribute.for_method
end

let copy_code_value cv =
  let copy = function
    | `Unknown (x, y) -> `Unknown (x, String.copy y)
    | x -> x in
  { cv with Attribute.attributes = List.map copy cv.Attribute.attributes }

class default_class_definition_mapper : class_definition_mapper = object (self)
  method class_definition flags name extends implements fields methods attributes =
    let flags' = self#class_flags flags in
    let name' = self#class_name name in
    let extends' = self#class_extends extends in
    let implements' = List.map self#class_implements implements in
    let fields' = List.map (fun (x, y, z, t) -> self#class_field x y z t) fields in
    let methods' = List.map self#class_method methods in
    let attributes' = List.map self#class_attribute attributes in
    (flags', name', extends', implements', fields', methods', attributes')

  method class_flags x = x

  method class_name x = x

  method class_extends x = x

  method class_implements x = x

  method class_field flags name descriptor attributes =
    let flags' = self#field_flags flags in
    let name' = self#field_name name in
    let descriptor' = self#field_descriptor descriptor in
    let attributes' = List.map self#field_attribute attributes in
    (flags', name', descriptor', attributes')

  method class_method = function
    | Method.Regular (flags, name, descriptor, attributes) ->
        let x, y, z, t = self#regular_method flags name descriptor attributes in
        Method.Regular (x, y, z, t)
    | Method.Constructor (flags, descriptor, attributes) ->
        let x, y, z = self#constructor_method flags descriptor attributes in
        Method.Constructor (x, y, z)
    | Method.Initializer (strictfp, attributes) ->
        let x, y = self#initializer_method strictfp attributes in
        Method.Initializer (x, y)

  method class_attribute = function
    | `Unknown (x, y) -> `Unknown (x, String.copy y)
    | x -> x

  method field_flags x = x

  method field_name x = x

  method field_descriptor x = x

  method field_attribute = function
    | `Unknown (x, y) -> `Unknown (x, String.copy y)
    | x -> x

  method regular_method flags name descriptor attributes =
    let flags' = self#regular_method_flags flags in
    let name' = self#regular_method_name name in
    let descriptor' = self#regular_method_descriptor descriptor in
    let attributes' = List.map self#regular_method_attribute attributes in
    (flags', name', descriptor', attributes')

  method regular_method_flags x = x

  method regular_method_name x = x

  method regular_method_descriptor x = x

  method regular_method_attribute = function
    | `Code cv -> `Code (copy_code_value cv)
    | `Unknown (x, y) -> `Unknown (x, String.copy y)
    | x -> x

  method constructor_method flags descriptor attributes =
    let flags' = self#constructor_method_flags flags in
    let descriptor' = self#constructor_method_descriptor descriptor in
    let attributes' = List.map self#constructor_method_attribute attributes in
    (flags', descriptor', attributes')

  method constructor_method_flags x = x

  method constructor_method_descriptor x = x

  method constructor_method_attribute = function
    | `Code cv -> `Code (copy_code_value cv)
    | `Unknown (x, y) -> `Unknown (x, String.copy y)
    | x -> x

  method initializer_method strictfp attributes =
    let strictfp' = self#initializer_method_strictfp strictfp in
    let attributes' = List.map self#initializer_method_attribute attributes in
    (strictfp', attributes')

  method initializer_method_strictfp x = x

  method initializer_method_attribute = function
    | `Code cv -> `Code (copy_code_value cv)
    | `Unknown (x, y) -> `Unknown (x, String.copy y)
    | x -> x
end

class type class_definition_iterator = object
  method class_definition : AccessFlag.for_class list -> Name.for_class -> Name.for_class option -> Name.for_class list -> Field.t list -> Method.t list -> Attribute.for_class list -> unit
  method class_flags : AccessFlag.for_class list -> unit
  method class_name : Name.for_class -> unit
  method class_extends : Name.for_class option -> unit
  method class_implements : Name.for_class -> unit
  method class_field : AccessFlag.for_field list -> Name.for_field -> Descriptor.for_field -> Attribute.for_field list -> unit
  method class_method : Method.t -> unit
  method class_attribute : Attribute.for_class -> unit
  method field_flags : AccessFlag.for_field list -> unit
  method field_name : Name.for_field -> unit
  method field_descriptor : Descriptor.for_field -> unit
  method field_attribute : Attribute.for_field -> unit
  method regular_method : AccessFlag.for_method list -> Name.for_method -> Descriptor.for_method -> Attribute.for_method list -> unit
  method regular_method_flags : AccessFlag.for_method list -> unit
  method regular_method_name : Name.for_method -> unit
  method regular_method_descriptor : Descriptor.for_method -> unit
  method regular_method_attribute : Attribute.for_method -> unit
  method constructor_method : AccessFlag.for_constructor list -> Descriptor.for_parameter list -> Attribute.for_method list -> unit
  method constructor_method_flags : AccessFlag.for_constructor list -> unit
  method constructor_method_descriptor : Descriptor.for_parameter list -> unit
  method constructor_method_attribute : Attribute.for_method -> unit
  method initializer_method : bool -> Attribute.for_method list -> unit
  method initializer_method_strictfp : bool -> unit
  method initializer_method_attribute : Attribute.for_method -> unit
end

class default_class_definition_iterator : class_definition_iterator = object (self)
  method class_definition flags name extends implements fields methods attributes =
    self#class_flags flags;
    self#class_name name;
    self#class_extends extends;
    List.iter self#class_implements implements;
    List.iter (fun (x, y, z, t) -> self#class_field x y z t) fields;
    List.iter self#class_method methods;
    List.iter self#class_attribute attributes

  method class_flags _ = ()

  method class_name _ = ()

  method class_extends _ = ()

  method class_implements _ = ()

  method class_field flags name descriptor attributes =
    self#field_flags flags;
    self#field_name name;
    self#field_descriptor descriptor;
    List.iter self#field_attribute attributes

  method class_method = function
    | Method.Regular (flags, name, descriptor, attributes) ->
        self#regular_method flags name descriptor attributes
    | Method.Constructor (flags, descriptor, attributes) ->
        self#constructor_method flags descriptor attributes
    | Method.Initializer (strictfp, attributes) ->
        self#initializer_method strictfp attributes

  method class_attribute _ = ()

  method field_flags _ = ()

  method field_name _ = ()

  method field_descriptor _ = ()

  method field_attribute _ = ()

  method regular_method flags name descriptor attributes =
    self#regular_method_flags flags;
    self#regular_method_name name;
    self#regular_method_descriptor descriptor;
    List.iter self#regular_method_attribute attributes

  method regular_method_flags _ = ()

  method regular_method_name _ = ()

  method regular_method_descriptor _ = ()

  method regular_method_attribute _ = ()

  method constructor_method flags descriptor attributes =
    self#constructor_method_flags flags;
    self#constructor_method_descriptor descriptor;
    List.iter self#constructor_method_attribute attributes

  method constructor_method_flags _ = ()

  method constructor_method_descriptor _ = ()

  method constructor_method_attribute _ = ()

  method initializer_method strictfp attributes =
    self#initializer_method_strictfp strictfp;
    List.iter self#initializer_method_attribute attributes

  method initializer_method_strictfp _ = ()

  method initializer_method_attribute _ = ()
end
