/*
 * This file is part of Barista.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.FileOutputStream;
import java.util.Collections;
import java.util.List;
import static java.util.Arrays.asList;

import fr.x9c.barista.api.*;
import static fr.x9c.barista.api.Instruction.*;
import static tests.Common.*;

public final class Source {

    private static final FieldRef systOut =
	new FieldRef("java.lang.System",
		     "out",
		     new Descriptor.Class("java.io.PrintStream"));

    private static final MethodRef println =
	new MethodRef("java.io.PrintStream",
		      "println",
		      Descriptor.VOID,
		      descs(new Descriptor.Class("java.lang.String")));

    private static final MethodRef print =
	new MethodRef("java.io.PrintStream",
		      "print",
		      Descriptor.VOID,
		      descs(new Descriptor.Class("java.lang.String")));

    public static void main(final String[] args) throws Throwable {
	final Field prefix = new Field(asList(AccessFlag.Private,
					     AccessFlag.Static,
					     AccessFlag.Final),
				       "PREFIX",
				       new Descriptor.Class("java.lang.String"),
				       Collections.<Attribute>singletonList(new Attribute.ConstantValue(new Attribute.StringValue("  - << "))));
	final Field suffix = new Field(asList(AccessFlag.Private,
					     AccessFlag.Static,
					     AccessFlag.Final),
				       "SUFFIX",
				       new Descriptor.Class("java.lang.String"),
				       Collections.<Attribute>singletonList(new Attribute.ConstantValue(new Attribute.StringValue(" >>"))));
	final List<Instruction> printInstructions =
	    asList(new GETSTATIC(systOut),
		   DUP,
		   DUP,
		   new GETSTATIC(new FieldRef("pack.Test",
					      "PREFIX",
					      new Descriptor.Class("java.lang.String"))),
		   new INVOKEVIRTUAL(print),
		   ALOAD_0,
		   new INVOKEVIRTUAL(print),
		   new GETSTATIC(new FieldRef("pack.Test",
					      "SUFFIX",
					      new Descriptor.Class("java.lang.String"))),
		   new INVOKEVIRTUAL(println),
		   RETURN);
	final List<Instruction> instructions =
	    asList(NOP,
		   new GETSTATIC(systOut),
		   new LDC_STRING("hello\t... \n\t... \"world\""),
		   new INVOKEVIRTUAL(println),
		   ICONST_0,
		   ISTORE_1,
		   ALOAD_0,
		   ARRAYLENGTH,
		   ISTORE_2,
		   /* loop: */
		   ILOAD_1,
		   ILOAD_2,
		   new IF_ICMPEQ((short) 15),
		   ALOAD_0,
		   ILOAD_1,
		   AALOAD,
		   new INVOKESTATIC(new MethodRef("pack.Test",
						  "print",
						  Descriptor.VOID,
						  descs(new Descriptor.Class("java.lang.String")))),
		   new IINC((short) 1, (byte) 1),
		   new GOTO((short) -14),
		   /* end: */
		   RETURN);
	final Method print = compileMethod(asList(AccessFlag.Public, AccessFlag.Static),
					   "print",
					   Descriptor.VOID,
					   descs(new Descriptor.Class("java.lang.String")),
					   4,
					   4,
					   Collections.<Attribute.ExceptionTableElement>emptyList(),
					   Collections.<Attribute>emptyList(),
					   Collections.<Attribute>emptyList(),
					   printInstructions);
	final Method main = compileMethod(instructions);
	final ClassDefinition cd = compileClass(null,
						null,
						null,
						null,
						asList(prefix, suffix),
						null,
						asList(print, main));
	final byte[] bc = ByteCode.encode(cd);
	FileOutputStream out = new FileOutputStream("pack/Test.class");
	out.write(bc);
	out.flush();
    }

}
