/*
 * This file is part of Barista.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.FileOutputStream;
import java.util.Collections;
import java.util.List;
import static java.util.Arrays.asList;

import fr.x9c.barista.api.*;
import static fr.x9c.barista.api.Instruction.*;
import static tests.Common.*;

public final class Source {

    private static final FieldRef systOut =
	new FieldRef("java.lang.System",
		     "out",
		     new Descriptor.Class("java.io.PrintStream"));

    private static final MethodRef println =
	new MethodRef("java.io.PrintStream",
		      "println",
		      Descriptor.VOID,
		      descs(new Descriptor.Class("java.lang.String")));

    private static final ArrayMethodRef toStringIntArray =
	new ArrayMethodRef(new Descriptor.Array(Descriptor.INT),
		      "toString",
		      new Descriptor.Class("java.lang.String"),
		      descs());

    public static void main(final String[] args) throws Throwable {
	final List<Instruction> instructions =
	    asList(/* interface */
		   new GETSTATIC(systOut),
		   new LDC_STRING("interface"),
		   new INVOKEINTERFACE(new MethodRef("java.lang.Appendable",
						     "append",
						     new Descriptor.Class("java.lang.Appendable"),
						     descs(new Descriptor.Class("java.lang.CharSequence"))),
				       (short) 2),

		   /* special */
		   new GETSTATIC(systOut),
		   new NEW(new TypeName("java.lang.Integer")),
		   DUP,
		   new LDC_STRING("012"),
		   new INVOKESPECIAL(new MethodRef("java.lang.Integer",
						   "<init>",
						   Descriptor.VOID,
						   descs(new Descriptor.Class("java.lang.String")))),
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(new Descriptor.Class("java.lang.Object")))),

		   /* static */
		   new GETSTATIC(systOut),
		   new LDC_STRING("123"),
		   new INVOKESTATIC(new MethodRef("java.lang.Integer",
						  "parseInt",
						  Descriptor.INT,
						  descs(new Descriptor.Class("java.lang.String")))),
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   /* virtual (over array type) */
                   ICONST_3,
                   new NEWARRAY(Descriptor.INT),
                   new INVOKEVIRTUAL_ARRAY(toStringIntArray),
                   POP,

		   /* virtual (over class) */
		   new GETSTATIC(systOut),
		   new LDC_STRING("virtual"),
		   new INVOKEVIRTUAL(println),
		   RETURN);
	final Method main = compileMethod(asList(AccessFlag.Public, AccessFlag.Static),
					  "main",
					  Descriptor.VOID,
					  descs(new Descriptor.Array(new Descriptor.Class("java.lang.String"))),
					  5,
					  4,
					  Collections.<Attribute.ExceptionTableElement>emptyList(),
					  Collections.<Attribute>emptyList(),
					  Collections.<Attribute>emptyList(),
					  instructions);
	final ClassDefinition cd = compileClass(null,
						null,
						null,
						null,
						null,
						null,
						asList(main));
	final byte[] bc = ByteCode.encode(cd);
	FileOutputStream out = new FileOutputStream("pack/Test.class");
	out.write(bc);
	out.flush();
    }

}
