/*
 * This file is part of Barista.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.FileOutputStream;
import java.util.Collections;
import java.util.List;
import static java.util.Arrays.asList;

import fr.x9c.barista.api.*;
import static fr.x9c.barista.api.Instruction.*;
import static tests.Common.*;

public final class Source {

    private static final FieldRef systOut =
	new FieldRef("java.lang.System",
		     "out",
		     new Descriptor.Class("java.io.PrintStream"));

    private static final MethodRef println =
	new MethodRef("java.io.PrintStream",
		      "println",
		      Descriptor.VOID,
		      descs(new Descriptor.Class("java.lang.String")));

    public static void main(final String[] args) throws Throwable {
	final List<Instruction> tableInstructions =
	    asList(ICONST_0,
		   ISTORE_1,
		   /* loop: */
		   ILOAD_1,
		   DUP,
		   new LDC_INT(5),
		   new IF_ICMPEQ((short) 73), /* end */
		   new TABLESWITCH(48, 0, 2, asList(27, 34, 41)),
		   /* zero: */
		   new LDC_STRING("zero"),
		   new GOTO_W(26), /* print */
		   /* once: */
		   new LDC_STRING("once"),
		   new GOTO_W(19), /* print */
		   /* twice: */
		   new LDC_STRING("twice"),
		   new GOTO_W(12), /* print */
		   /* many: */
		   new LDC_STRING("many"),
		   new GOTO_W(5), /* print */
		   /* print: */
		   new GETSTATIC(systOut),
		   SWAP,
		   new INVOKEVIRTUAL(println),
		   new IINC ((short) 1, (byte) 1),
		   new GOTO_W (-72), /* loop */
		   /* end: */
		   RETURN);
	final List<Instruction> lookupInstructions =
	    asList(ICONST_0,
		   ISTORE_1,
		   /* loop: */
		   ILOAD_1,
		   DUP,
		   new LDC_INT(5),
		   new IF_ICMPEQ((short) 71), /* end */
		   new LOOKUPSWITCH(50, 3, asList(new MatchOffsetPair(0, 35),
						  new MatchOffsetPair(1, 40),
						  new MatchOffsetPair(2, 45))),
		   /* zero: */
		   new LDC_STRING("zero"),
		   new GOTO((short) 18), /* print */
		   /* once: */
		   new LDC_STRING("once"),
		   new GOTO((short) 13), /* print */
		   /* twice: */
		   new LDC_STRING("twice"),
		   new GOTO((short) 8), /* print */
		   /* many: */
		   new LDC_STRING("many"),
		   new GOTO((short) 3), /* print */
		   /* print: */
		   new GETSTATIC(systOut),
		   SWAP,
		   new INVOKEVIRTUAL(println),
		   new IINC ((short) 1, (byte) 1),
		   new GOTO ((short) -72), /* loop */
		   /* end: */
		   RETURN);
	final List<Instruction> instructions =
	    asList(new INVOKESTATIC(new MethodRef("pack.Test",
						  "table",
						  Descriptor.VOID,
						  descs())),
		   new GETSTATIC(systOut),
		   new LDC_STRING("---"),
		   new INVOKEVIRTUAL(println),
		   new INVOKESTATIC(new MethodRef("pack.Test",
						  "lookup",
						  Descriptor.VOID,
						  descs())),
		   RETURN);
	final Method table =
	    compileMethod(asList(AccessFlag.Public, AccessFlag.Static),
			  "table",
			  Descriptor.VOID,
			  descs(),
			  4,
			  4,
			  Collections.<Attribute.ExceptionTableElement>emptyList(),
			  Collections.<Attribute>emptyList(),
			  Collections.<Attribute>emptyList(),
			  tableInstructions);
	final Method lookup =
	    compileMethod(asList(AccessFlag.Public, AccessFlag.Static),
			  "lookup",
			  Descriptor.VOID,
			  descs(),
			  4,
			  4,
			  Collections.<Attribute.ExceptionTableElement>emptyList(),
			  Collections.<Attribute>emptyList(),
			  Collections.<Attribute>emptyList(),
			  lookupInstructions);
	final Method main = compileMethod(instructions);
	final ClassDefinition cd = compileClass(null,
						null,
						null,
						null,
						null,
						null,
						asList(table,
						       lookup,
						       main));
	final byte[] bc = ByteCode.encode(cd);
	FileOutputStream out = new FileOutputStream("pack/Test.class");
	out.write(bc);
	out.flush();
    }

}
